
/*
   make_log.c - making & updating history files, counting summary 
   $Id: make_log.c,v 1.5 2000/06/26 14:18:37 yossa Exp $
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <libintl.h>
#include <locale.h>
#include <zlib.h>
#include "monopoly.h"
#include "mconf.h"
#include "make_log.h"
#include "intl.h"

int currentYear()
{
    time_t t;
    struct tm *ts;

    t = time(NULL);
    ts = localtime(&t);
    return ts->tm_year;
}

void getConnectionCost(char *start, char *stop,
	     unsigned int *hours, unsigned int *mins, unsigned int *secs,
		       unsigned int *impulses, float *cost)
{
    char ss[20];
    struct tm ts;
    int month;
    int year;
    time_t StartTime, StopTime;
    time_t diff;

    time_t stoptime, st;
    unsigned int ilen, icount;
    float icost;

    year = currentYear();
    strncpy(ss, stop, 15);
    strptime(ss, "%h %e %T", &ts);
    month = ts.tm_mon;
    ts.tm_year = year;		/* current year */

    StopTime = mktime(&ts);
    strncpy(ss, start, 15);
    strptime(ss, "%h %e %T", &ts);
    /* is it previous year? really dirty solution, needs to be changed */
    ts.tm_year = ((ts.tm_mon > month) ? year - 1 : year);
    StartTime = mktime(&ts) - DelayTime;
    diff = difftime(StopTime, StartTime);
    *hours = (diff / TTIME_1HOUR);
    *mins = (diff - (*hours) * TTIME_1HOUR) / TTIME_1MIN;
    *secs = (diff % TTIME_1MIN);
    *impulses = 0;
    *cost = 0;

    while (diff > 0) {
	getActiveRule(StartTime, &ilen, &icost, &stoptime);
	st = (StopTime < stoptime) ? StopTime : stoptime;
	icount = (st - StartTime) / ilen + 1;	/* ???? */
	*impulses += icount;
	*cost += icount * icost;
	StartTime += icount * ilen;
	diff = difftime(StopTime, StartTime);
    }
}

char *removeInfo(char *line)
{
    char *spptr;
    spptr = strstr(line, ":");
    spptr++;
    spptr = strstr(spptr, ":");
    spptr++;
    spptr = strstr(spptr, ":");
    spptr++;
    spptr = strtok(spptr, "^M");
    spptr += 2;
    spptr = strtok(spptr, "/");
    spptr[8] = '\0';
    return spptr;
}

gzFile *myopen(char *name,char*mode)
{
    gzFile *f;
    char buf[256];
    f=gzopen(name,mode);
    if (f==NULL) {
	strncpy(buf,name,256);
	strncat(buf,".gz",256);
	f=gzopen(buf,mode);
    }	
    return f;
}

void updateLogFile(int all)
{
    char StartTime[] = "        \0", StopTime[] = "        \0", LastConnection[255];
    char speed[256];
    char StartLine[255], StopLine[255], LogLine[41] = "";
    unsigned int Impulses, Hours, Minutes, Seconds, RepeatedLine_DontAppend = 1,
     Sp = 0, pass = 0;
    char *spptr;
    int fnum;
    char name[256];
    char suf[256];
    int was_speed;
    gzFile *messages;
    FILE *log;
    float ConnectionCost;

    if (all) {
	printf("%s\n", _("Please wait... creating new history file..."));
	pass = 1;
	RepeatedLine_DontAppend = 0;
    } else
	printf("%s\n", _("Please wait... updating history file..."));

    while (pass != 2) {
	if ((messages = myopen(SystemLog, "r")) == NULL)
	    done(1, _("I need permisions to read [SystemLog]\nType chmod "
		      "o+r [SystemLog]\n  (It will make your system logs "
		      "visible to everybody!)\n\n[SystemLog]"));
	if (messages)
	    gzclose(messages);

	if (!all) {
	    if ((log = fopen(LogFileName, "rt")) == NULL)
		done(1, LogFileName);
	    fseek(log, -120, SEEK_END);
	    while (!feof(log))
		fgets(LastConnection, 255, log);
	    fclose(log);
	    if ((log = fopen(LogFileName, "at")) == NULL)
		done(1, LogFileName);
	} else if ((log = fopen(LogFileName, "wt")) == NULL)
	    done(1, LogFileName);


	for (fnum = 10; fnum >= 0; fnum--) {
	    strcpy(name, SystemLog);
	    if (fnum) {
		sprintf(suf, ".%i", fnum);
		strcat(name, suf);
	    }
	    if ((messages = myopen(name, "r")) == NULL)
		continue;

	    if (!pass) {
		gzgets(messages,StartLine, 255);
		while (strncmp(StartLine, LastConnection, 15) != 0 && !gzeof(messages))
		    gzgets(messages, StartLine, 255);
		if (gzeof(messages)) {
		    gzclose(messages);
		    continue;
		}
	    }
	    pass = 1;
	    while (!gzeof(messages)) {
		strcpy(speed, "-       ");
		was_speed = 0;
		gzgets(messages, StartLine, 255);
		if (gzeof(messages))
		    continue;
		if (strstr(StartLine, StartMsg) != NULL
		    && strstr(StartLine, StartProgram) != NULL && !gzeof(messages)) {
		    gzgets(messages, StopLine, 255);
		    while (!gzeof(messages)) {
			if (strstr(StopLine, StartMsg) != NULL
			    && strstr(StopLine, StartProgram) != NULL)
			    strcpy(StartLine, StopLine);
			if (Sp && !was_speed) {
			    strncpy(speed, removeInfo(StopLine), 8);
			    speed[8] = '\0';
			    while (strlen(speed) < 8)
				strcat(speed, " ");
			    was_speed = 1;
			}
			if (strstr(StopLine, SpeedAfter) != NULL)
			    Sp = 1;
			else
			    Sp = 0;

			gzgets(messages, StopLine, 255);
			if (strstr(StopLine, EndMsg) != NULL
			    && strstr(StopLine, EndProgram) != NULL)
			    break;
			if (gzeof(messages)) {
			    while (fnum) {
				fnum--;
				strcpy(name, SystemLog);
				if (fnum) {
				    sprintf(suf, ".%i", fnum);
				    strcat(name, suf);
				}
				gzclose(messages);
				if ((messages = myopen(name, "r")) == NULL) {
				    if (!fnum)
					done(1, _("I need permisions to read "
					       "[SystemLog]\nType chmod "
					   "o+r [SystemLog]\n  (It will "
						  "make your system logs "
					     "visible to everybody!)\n\n"
						  "[SystemLog]"));


				}
				break;
			    }
			}
		    }
		    strcpy(LogLine, "\0");
		    strncat(LogLine, StartLine, 15);
		    strncpy(StartTime, StartLine + 7, 8);
		    strncpy(StopTime, StopLine + 7, 8);
		    /* count connection cost */
		    Impulses = 0;
		    Hours = 0;
		    Minutes = 0;
		    Seconds = 0;
		    getConnectionCost(StartLine, StopLine,
				      &Hours, &Minutes, &Seconds, &Impulses, &ConnectionCost);
		    if (!gzeof(messages) && !RepeatedLine_DontAppend)
			fprintf(log, "%s   %.2i:%.2i:%.2i   %.3i   %.2f   %s\n",
			      LogLine, Hours, Minutes, Seconds, Impulses,
				ConnectionCost, speed);
		    else
			RepeatedLine_DontAppend = 0;

		}
	    }
	    gzclose(messages);
	}
	fclose(log);
	RepeatedLine_DontAppend = 0;
	if (!pass)
	    pass = 1;
	else
	    pass = 2;
    }
}

unsigned int GetNumberOfMonth(char MonthName[3])
{
    char MonthTable[12][3] =
    {
	{'J', 'a', 'n'},
	{'F', 'e', 'b'},
	{'M', 'a', 'r'},
	{'A', 'p', 'r'},
	{'M', 'a', 'y'},
	{'J', 'u', 'n'},
	{'J', 'u', 'l'},
	{'A', 'u', 'g'},
	{'S', 'e', 'p'},
	{'O', 'c', 't'},
	{'N', 'o', 'v'},
	{'D', 'e', 'c'}};
    int i;

    for (i = 0; i <= 11; i++) {
	if (strncmp(MonthTable[i], MonthName, 3) == 0)
	    return i + 1;
    }
    return 0;
}

AddPrice(char *Line, unsigned long int *SecondsSum,
	 unsigned long int *ImpulsSum, float *CostSum)
{
    *SecondsSum += atol(Line + 18) * 3600;
    *SecondsSum += atol(Line + 21) * 60;
    *SecondsSum += atol(Line + 24);
    *ImpulsSum += atol(Line + 27);
    *CostSum += atof(Line + 33);
}

void countConnections(unsigned int StartDay, unsigned int StartMonth, 
		      unsigned int StopDay, unsigned int StopMonth, 
		      TDialSummary * DialSummary)
{
    char Line[255], PrevLine[255], MonthName[3];
    unsigned int NumberOfMonth, NumberOfDay;
    unsigned long int SecondsSum = 0, ImpulsSum = 0;
    float CostSum = 0.0;
    FILE *log;

    if ((log = fopen(LogFileName, "r")) == NULL)
	done(1, LogFileName);
    while (!feof(log)) {
	strcpy(PrevLine, Line);
	fgets(Line, 255, log);
	if (feof(log))
	    continue;
	strncpy(MonthName, Line, 3);
	NumberOfMonth = GetNumberOfMonth(MonthName);
	NumberOfDay = atoi(Line + 4);
	if ((NumberOfMonth > StartMonth && NumberOfMonth < StopMonth) \
	    ||(NumberOfMonth == StartMonth && NumberOfDay >= StartDay && NumberOfMonth < StopMonth) \
	    ||(NumberOfMonth == StartMonth && NumberOfDay >= StartDay && NumberOfMonth == StopMonth && NumberOfDay <= StopDay) \
	    ||(NumberOfMonth > StartMonth && NumberOfMonth == StopMonth && NumberOfDay <= StopDay) \
	    &&!feof(log)) {
	    if (strcmp(Line, PrevLine))
		AddPrice(Line, &SecondsSum, &ImpulsSum, &CostSum);
	}
    }
    (*DialSummary).days = SecondsSum / 86400;
    SecondsSum -= (*DialSummary).days * 86400;
    (*DialSummary).hours = SecondsSum / 3600;
    SecondsSum -= (*DialSummary).hours * 3600;
    (*DialSummary).hours += (*DialSummary).days * 24;
    (*DialSummary).days = 0;
    (*DialSummary).minutes = SecondsSum / 60;
    SecondsSum -= (*DialSummary).minutes * 60;
    (*DialSummary).seconds = SecondsSum;
    (*DialSummary).impulses = ImpulsSum;
    (*DialSummary).price = CostSum;
    fclose(log);
}
void countLastConnection(TDialSummary * DialSummary)
{
    char Line[255];
    unsigned long int SecondsSum = 0, ImpulsSum = 0;
    float CostSum = 0.0;
    FILE *log;

    if ((log = fopen(LogFileName, "r")) == NULL)
	done(1, LogFileName);
    fseek(log, -51, SEEK_END);
    while (!feof(log))
	fgets(Line, 255, log);
    AddPrice(Line, &SecondsSum, &ImpulsSum, &CostSum);
    (*DialSummary).days = SecondsSum / 86400;
    SecondsSum -= (*DialSummary).days * 86400;
    (*DialSummary).hours = SecondsSum / 3600;
    SecondsSum -= (*DialSummary).hours * 3600;
    (*DialSummary).hours += (*DialSummary).days * 24;
    (*DialSummary).days = 0;
    (*DialSummary).minutes = SecondsSum / 60;
    SecondsSum -= (*DialSummary).minutes * 60;
    (*DialSummary).seconds = SecondsSum;
    (*DialSummary).impulses = ImpulsSum;
    (*DialSummary).price = CostSum;
    fclose(log);
}
