/*  $Id: mconf.c,v 1.3 1999/10/16 13:58:14 yossa Exp $ */

#include <stdio.h>
#include <string.h>
#include "mconf.h"
#include "intl.h"

float ImpulsPrice;
int ImpulsAfter;
int DelayTime;
char LogFileName[256];
char StartProgram[256], StartMsg[256], EndProgram[256], EndMsg[256],
 SpeedAfter[256], SystemLog[256];

struct RULE {
    float cost;
    int len;
    time_t from, until;
    struct RULE *next;
};

struct RULE *RuleHead = NULL;

static void setConfigDefaults()
{
    strcpy(SystemLog, "/var/log/messages");
    ImpulsAfter = 180;
    DelayTime = 0;
    ImpulsPrice = 0.23;
}

int setVariable(char *id, char *value)
{
    if (!strcmp(id, "AddDelay"))
	sscanf(value, "%i", &DelayTime);
    else if (!strcmp(id, "LogFile"))
	strcpy(LogFileName, value);
    else if (!strcmp(id, "SystemLog"))
	strcpy(SystemLog, value);
    else if (!strcmp(id, "StartProgram"))
	strcpy(StartProgram, strtok(strstr(value, "\""), "\""));
    else if (!strcmp(id, "StartMsg"))
	strcpy(StartMsg, strtok(strstr(value, "\""), "\""));
    else if (!strcmp(id, "EndProgram"))
	strcpy(EndProgram, strtok(strstr(value, "\""), "\""));
    else if (!strcmp(id, "EndMsg"))
	strcpy(EndMsg, strtok(strstr(value, "\""), "\""));
    else if (!strcmp(id, "SpeedAfter"))
	strcpy(SpeedAfter, strtok(strstr(value, "\""), "\""));
    else
	return M_ERR_UNKNOWN_VARIABLE;
    return M_ERR_OK;
}

void getActiveRule(time_t start, unsigned int *itime, float *icost,
		   time_t * stop)
{
    struct tm ts;
    time_t t;
    struct RULE *p = RuleHead;

    memcpy(&ts, localtime(&start), sizeof(ts));
    t = TTIME_1HOUR * ts.tm_hour + TTIME_1MIN * ts.tm_min + ts.tm_sec;
    while (p) {
	if ((t >= p->from) && (t < p->until))
	    break;
	p = p->next;
    }
    if (p) {
	t = p->until;
	*icost = p->cost;
	*itime = p->len;
	ts.tm_hour = t / TTIME_1HOUR;
	ts.tm_min = (t - ts.tm_hour * TTIME_1HOUR) / TTIME_1MIN;
	ts.tm_sec = t % TTIME_1MIN;
	*stop = mktime(&ts);
    } else {
	printf("%s", _("mconf.c: get_active_rule(): no rule found, I'm embarrassed\n"));
	abort();
	/* co si spieprzyo -- tak nie ma prawa si sta */
    }
}

static struct RULE *createRule(int from, int until, int ilen, float cost)
{
    struct RULE *p;
    p = (struct RULE *) malloc(sizeof(struct RULE));
    p->next = NULL;
    p->from = from;
    p->until = until;
    p->len = ilen;
    p->cost = cost;
    return p;
}

static int _addRule(int from, int until, int ilen, float cost)
{
    struct RULE *newrule, *p;

    newrule = createRule(from, until, ilen, cost);
    /* no rule validity check right now. Checks for conditions like: "only 
       non-intersecting rules are valid" should be added */
    if (RuleHead == NULL)
	RuleHead = newrule;
    else if (RuleHead->from >= newrule->until) {
	newrule->next = RuleHead;
	RuleHead = newrule;
    } else {
	p = RuleHead;
	while (p != NULL) {
	    if (p->until <= newrule->from)
		if ((p->next == NULL) ||
		((p->next != NULL) && (p->next->from >= newrule->until)))
		    break;
	    p = p->next;
	}
	if (p == NULL) {	/* error */
	    newrule->next = RuleHead;
	    RuleHead = newrule;
	    return M_ERR_INVALID_RULE;
	}
	newrule->next = p->next;
	p->next = newrule;
    }
    return M_ERR_OK;
}

/* adds a rule to the rules list */
int addRule(int from, int until, int ilen, float cost)
{
    int result;

    if (from < until)
	return _addRule(from, until, ilen, cost);
    result = _addRule(from, TTIME_1DAY, ilen, cost);
    if (result == M_ERR_OK) {
	if (until > 0)
	    result = _addRule(0, until, ilen, cost);
    }
    return result;
}

static void fillRulesList()
{
    /* fill gaps between rules, using default rate */
    struct RULE *p, *newrule;

    if (RuleHead == NULL)
	RuleHead = createRule(0, TTIME_1DAY, ImpulsAfter, ImpulsPrice);
    else {
	if (RuleHead->from > 0) {
	    newrule = createRule(0, RuleHead->from, ImpulsAfter, ImpulsPrice);
	    newrule->next = RuleHead;
	    RuleHead = newrule;
	}
	p = RuleHead;
	while (p != NULL) {
	    if (p->next != NULL) {
		if (p->next->from != p->until) {
		    newrule = createRule(p->until, p->next->from, ImpulsAfter, ImpulsPrice);
		    newrule->next = p->next;
		    p->next = newrule;
		}
	    } else {
		if (p->until < TTIME_1DAY) {
		    newrule = createRule(p->until, TTIME_1DAY, ImpulsAfter, ImpulsPrice);
		    p->next = newrule;
		}
	    }
	    p = p->next;
	}
    }
}

extern FILE *yyin;
int yyparse();

int readConfigFile()
{
    FILE *f;
    char line[1000];
    int pass, read;

    setConfigDefaults();
    for (pass = 0, read = 0; pass < 2; pass++) {
	if (pass == 0) {
	    if ((f = fopen(CONFIGDIR, "rt")) == NULL)
		continue;
	} else if (pass == 1) {
	    if ((f = fopen(".monopolyrc", "rt")) == NULL)
		break;
	}
	read = 1;
	/* parse file */
	yyin = f;
	if (yyparse() != 0)
	    done(3, _("Error parsing config file"));	/* temp */
	fclose(f);
	/* parse file end */
    }
    if (!read)
	done(1, _("Cannot find monopolyrc"));
    fillRulesList();
}

void configCleanup()
{
    struct RULE *p;

    while (RuleHead) {
	p = RuleHead->next;
	free(RuleHead);
	RuleHead = p;
    }
}
