
/*
   MONOPOLY - counts your modem calls

   Changes since ver. 1.2 by Jarek Woloszyn (yossa@pld.org.pl)

   monopoly.c - main part of the program - showing statistics

   $Id: monopoly.c,v 1.3 1999/10/16 13:58:14 yossa Exp $
 */

#include <stdio.h>
#include <slang/slang.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <signal.h>
#include <libintl.h>
#include <locale.h>
#include "make_log.h"
#include "monopoly.h"
#include "moreslang.h"
#include "mconf.h"
#include "intl.h"

#include "../config.h"

int DontUpdateLogFile = 0, VisualMonopolyInitiation = 1, ActiveWindow;
time_t CurrentTime_InSec;
struct tm *TimeStruct;
TDialSummary DialSummary;

char *buffer, Line[256], CurrentDir[1000];
unsigned long LogFileSize;

struct {
    long int StartLine, LinesNr;
} ViewerStruct;
struct {
    unsigned int month_nr;
} MonthStruct;
struct {
    unsigned int month_nr, day_nr;
} DayStruct;
struct {
    int start_day, start_month, stop_day, stop_month;
} DateStruct;

void done(int status, char *error_message)
{
    configCleanup();
    deinitColor();
    deinitSlang();
    switch (status) {
    case 0:
	printf(_("--== Monopoly ver. %s ==--\n"), VERSION);
	break;
    case 1:
	perror(error_message);
	break;
    case 2:
	printf("%s\n\n", error_message);
	break;
    }
    chdir(CurrentDir);
    exit(1);
}


long int getNumberOfLines(char *FileName)
{
    long int i = 0;
    char foo[256];
    FILE *f;
    if ((f = fopen(FileName, "rt")) == NULL)
	done(1, FileName);
    while (!feof(f)) {
	i++;
	fgets(foo, 256, f);
    }
    fclose(f);
    return i;
}

long fsizeof(char *FileName)
{
    FILE *f;
    if ((f = fopen(FileName, "r")) == NULL)
	done(1, FileName);
    fseek(f, 0, SEEK_END);
    return ftell(f);
    fclose(f);
}

void drawStatus();

int SLkp_getkey_extend()
{
    int ch, x, y;
    x = SLsmg_get_column();
    y = SLsmg_get_row();
    for (;;) {
	drawStatus();
	SLsmg_gotorc(y, x);
	SLsmg_refresh();
	if (SLang_input_pending(10)) {
	    ch = SLang_getkey();
	    if (ch == 033) {
		if (SLang_input_pending(2) == 0)
		    return 033;
	    }
	    SLang_ungetkey(ch);
	    return SLkp_getkey();
	}
    }
}

void SLkp_getnum(int len, char *str)
{
    int c = 0, i = 0;
    while (c != SL_KEY_ENTER && c != 13) {
	c = SLkp_getkey_extend();
	switch (c) {
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	    if (i < len) {
		SLsmg_write_char(c);
		str[i] = c;
		i++;
		str[i] = 0;
	    } else
		SLtt_beep();
	    break;
	case SL_KEY_ENTER:
	case 13:
	    break;
	case SL_KEY_BACKSPACE:
	    if (i > 0) {
		i--;
		str[i] = 0;
		SLsmg_gotorc(SLsmg_get_row(), SLsmg_get_column() - 1);
		SLsmg_write_char(' ');
		SLsmg_gotorc(SLsmg_get_row(), SLsmg_get_column() - 1);
	    } else
		SLtt_beep();
	    break;
	default:
	    SLtt_beep();
	}
    }
}


int loadBuffer()
{
    int i;
    FILE *f;
    long read;
    void *buffer1;

    if ((f = fopen(LogFileName, "rt")) == NULL)
	done(1, LogFileName);
    LogFileSize = fsizeof(LogFileName);
    if ((buffer1 = malloc(LogFileSize + 1)) != NULL) {
	read = fread(buffer1, 1, LogFileSize, f);
	buffer = (char *) buffer1;
	buffer[LogFileSize] = '\0';
	fclose(f);
	return 0;
    } else {
	fclose(f);
	done(1, "malloc");
    }
}

void getLine(char *Line, unsigned int LineNr)
{
    unsigned int i = 0, line_len = 39;
    unsigned long int L = 0;

    while (i != LineNr) {
	line_len = strcspn(buffer + L, "\n");
	strncpy(Line, buffer + L, line_len);
	Line[line_len] = '\0';
	i++;
	L += line_len + 1;
    }
}

void view()
{
    int i;

    Line[49] = '\0';
    SLsmg_set_color(COLOR_NORMAL);
    if (ViewerStruct.LinesNr >= 15) {
	SLsmg_gotorc(21, 30);
	SLsmg_printf("16+%.5i/%.5i",
		   ViewerStruct.StartLine + 1, ViewerStruct.LinesNr - 1);
	for (i = 2; i <= 17; i++) {
	    getLine(Line, ViewerStruct.StartLine + i - 1);
	    SLsmg_gotorc(i + 3, 3);
	    SLsmg_printf(Line);
	}
    } else {
	for (i = 2; i <= ViewerStruct.LinesNr; i++) {
	    getLine(Line, i - 1);
	    SLsmg_gotorc(i + 2, 3);
	    SLsmg_printf(Line);
	}
    }
    SLsmg_refresh();
}

char *fillSpace(int num, char *str)
{
    static char tmp[256];
    int i;
    strcpy(tmp, str);
    if (strlen(tmp) > num)
	tmp[num] = 0;
    else
	for (i = num - strlen(tmp); i > 0; i--)
	    strcat(tmp, " ");

    return tmp;
}

void drawStatus()
{
    char data[] = "                   \0";
    SLsmg_set_color(COLOR_NORMAL);
    SLsmg_draw_clear_box(22, 0, 3, 79);
    SLsmg_gotorc(23, 1);
    switch (ActiveWindow) {
    case 1:
	SLsmg_printf(fillSpace(55,
		   _("<- -> change month  <TAB> next win.  <q> exit.")));
	break;
    case 2:
	SLsmg_printf(fillSpace(55,
		      _("<- -> chng. day  <TAB> next win.  <q> exit.")));
	break;
    case 3:
	SLsmg_printf(fillSpace(55,
		  _("<ENTER> input dates  <TAB> next win.  <q> exit.")));
	break;
    case 4:
	SLsmg_printf(fillSpace(55,
		_("<UP/DN/PUP/PDN/HM/END>, <TAB> next win. <q> exit.")));
	break;
    case 5:
	SLsmg_set_color(COLOR_BOX_ACTIVE);
	SLsmg_printf(fillSpace(55, _("Are you sure (y/n) ?")));
	break;
    }
    SLsmg_set_color(COLOR_TABLE_TITLE);
    time(&CurrentTime_InSec);
    TimeStruct = localtime(&CurrentTime_InSec);
    strncpy(data, asctime(TimeStruct), 19);
    SLsmg_gotorc(23, 58);
    SLsmg_printf(data);
    SLsmg_set_color(COLOR_NORMAL);
    SLsmg_refresh();
}

void finishWork()
{
    char *str = _("y");
    int c, oldActiveWindow;
    SLsmg_set_color(COLOR_NORMAL);
    SLsmg_draw_clear_box(22, 0, 3, 79);
    SLsmg_gotorc(23, 1);
    SLsmg_set_color(COLOR_BOX_ACTIVE);
    SLsmg_printf(fillSpace(55, _("Are you sure (y/n) ?")));
    SLsmg_refresh();
    oldActiveWindow = ActiveWindow;
    ActiveWindow = 5;
    if ((c = SLkp_getkey_extend()) != str[0])
	drawStatus();
    else
	done(0, "");
    ActiveWindow = oldActiveWindow;
}

void drawViewer()
{
    int c;
    SLsmg_set_color(COLOR_NORMAL);
    SLsmg_draw_clear_box(3, 0, 19, 56);
    SLsmg_gotorc(3, 10);
    SLsmg_set_color(COLOR_BOX_ACTIVE);
    SLsmg_printf(_("Calls"));
    SLsmg_gotorc(4, 1);
    SLsmg_set_color(COLOR_TABLE_TITLE);
    SLsmg_printf(_("   Date   Time     Duration Impulses Cost   Speed"));
    SLsmg_refresh();
    c = 0;
    while (c != '\t') {
	if (c == 'q' || c == 'Q')
	    finishWork();
	view();
	if (!VisualMonopolyInitiation)
	    c = SLkp_getkey_extend();
	else
	    c = '\t';
	switch (c) {
	case SL_KEY_NPAGE:
	    ViewerStruct.StartLine += 15;
	    if (ViewerStruct.StartLine >= ViewerStruct.LinesNr - 16)
		\
		    ViewerStruct.StartLine = ViewerStruct.LinesNr - 17;
	    break;
	case SL_KEY_DOWN:
	    ViewerStruct.StartLine++;
	    if (ViewerStruct.StartLine == ViewerStruct.LinesNr - 16)
		\
		    ViewerStruct.StartLine = ViewerStruct.LinesNr - 17;
	    break;
	case SL_KEY_PPAGE:
	    ViewerStruct.StartLine -= 15;
	    if (ViewerStruct.StartLine <= -1)
		ViewerStruct.StartLine = 0;
	    break;
	case SL_KEY_UP:
	    ViewerStruct.StartLine--;
	    if (ViewerStruct.StartLine == -1)
		ViewerStruct.StartLine = 0;
	    break;
	case SL_KEY_HOME:
	    ViewerStruct.StartLine = 0;
	    break;
	case SL_KEY_END:
	    ViewerStruct.StartLine = ViewerStruct.LinesNr - 17;
	    break;
	}
    }
    SLsmg_gotorc(3, 10);
    SLsmg_set_color(COLOR_BOX_TITLE);
    SLsmg_printf(_("Calls"));
    SLsmg_refresh();
}

void getMonthName(unsigned int month_nr, char month_name[])
{
    switch (month_nr) {
    case 1:
	strcpy(month_name, _("Jan"));
	break;
    case 2:
	strcpy(month_name, _("Feb"));
	break;
    case 3:
	strcpy(month_name, _("Mar"));
	break;
    case 4:
	strcpy(month_name, _("Apr"));
	break;
    case 5:
	strcpy(month_name, _("May"));
	break;
    case 6:
	strcpy(month_name, _("Jun"));
	break;
    case 7:
	strcpy(month_name, _("Jul"));
	break;
    case 8:
	strcpy(month_name, _("Aug"));
	break;
    case 9:
	strcpy(month_name, _("Sep"));
	break;
    case 10:
	strcpy(month_name, _("Oct"));
	break;
    case 11:
	strcpy(month_name, _("Nov"));
	break;
    case 12:
	strcpy(month_name, _("Dec"));
	break;
    default:
	strcpy(month_name, "???");
	break;
    }
}



void drawMonth()
{
    int c;
    char MonthStr[10];
    c = 0;
    while (c != '\t') {
	if (c == 'q' || c == 'Q')
	    finishWork();
	getMonthName(MonthStruct.month_nr, MonthStr);
	SLsmg_set_color(COLOR_NORMAL);
	SLsmg_draw_clear_box(3, 57, 6, 21);
	SLsmg_gotorc(3, 61);
	SLsmg_set_color(COLOR_BOX_ACTIVE);
	SLsmg_printf("%s", MonthStr);

	countConnections(1, MonthStruct.month_nr, 33, MonthStruct.month_nr,
			 &DialSummary);

	SLsmg_set_color(COLOR_NORMAL);
	SLsmg_gotorc(4, 58);
	SLsmg_printf("%s: %i:%02i:%0i:%02i", _("Time"),
		     DialSummary.days, DialSummary.hours,
		     DialSummary.minutes, DialSummary.seconds);
	SLsmg_gotorc(5, 58);
	SLsmg_printf("%s: %i", _("Impulses"),
		     DialSummary.impulses);
	SLsmg_gotorc(6, 58);
	SLsmg_printf("%s: %.2f %s", _("Cost"),
		     DialSummary.price, _("$"));
	SLsmg_refresh();
	if (!VisualMonopolyInitiation)
	    c = SLkp_getkey_extend();
	else
	    c = '\t';
	switch (c) {
	case SL_KEY_LEFT:
	    MonthStruct.month_nr--;
	    if (MonthStruct.month_nr == 0)
		MonthStruct.month_nr = 12;
	    if (MonthStruct.month_nr == (*TimeStruct).tm_mon + 1)
		MonthStruct.month_nr = (*TimeStruct).tm_mon + 2;
	    if (MonthStruct.month_nr == 13)
		MonthStruct.month_nr = 1;
	    break;
	case SL_KEY_RIGHT:
	    MonthStruct.month_nr++;
	    if (MonthStruct.month_nr == (*TimeStruct).tm_mon + 2)
		MonthStruct.month_nr = (*TimeStruct).tm_mon + 1;
	    if (MonthStruct.month_nr == 13)
		MonthStruct.month_nr = 1;
	    break;
	}
    }
    SLsmg_gotorc(3, 61);
    SLsmg_set_color(COLOR_BOX_TITLE);
    SLsmg_printf("%s", MonthStr);
    SLsmg_refresh();
}

void drawDay()
{
    int c = 0;
    char MonthStr[10];
    while (c != '\t') {
	if (c == 'q' || c == 'Q')
	    finishWork();
	getMonthName(DayStruct.month_nr, MonthStr);
	SLsmg_set_color(COLOR_NORMAL);
	SLsmg_draw_clear_box(9, 57, 6, 21);
	SLsmg_gotorc(9, 61);
	SLsmg_set_color(COLOR_BOX_ACTIVE);
	SLsmg_printf("%s %i", MonthStr, DayStruct.day_nr);

	countConnections(DayStruct.day_nr, DayStruct.month_nr,
		     DayStruct.day_nr, DayStruct.month_nr, &DialSummary);

	SLsmg_set_color(COLOR_NORMAL);
	SLsmg_gotorc(10, 58);
	SLsmg_printf("%s: %i:%02i:%02i:%02i", _("Time"),
		     DialSummary.days, DialSummary.hours,
		     DialSummary.minutes, DialSummary.seconds);
	SLsmg_gotorc(11, 58);
	SLsmg_printf("%s: %i", _("Impulses"),
		     DialSummary.impulses);
	SLsmg_gotorc(12, 58);
	SLsmg_printf("%s: %.2f %s", _("Cost"),
		     DialSummary.price, _("$"));
	SLsmg_refresh();
	if (!VisualMonopolyInitiation)
	    c = SLkp_getkey_extend();
	else
	    c = '\t';
	switch (c) {
	case SL_KEY_LEFT:
	    DayStruct.day_nr--;
	    if (DayStruct.day_nr == 0) {
		DayStruct.day_nr = 31;
		DayStruct.month_nr--;
		if (DayStruct.month_nr == 0) {
		    DayStruct.day_nr = 31;
		    DayStruct.month_nr = 12;
		}
	    }
	    if (DayStruct.month_nr == (*TimeStruct).tm_mon + 1 &&
		DayStruct.day_nr == (*TimeStruct).tm_mday) {
		DayStruct.day_nr = (*TimeStruct).tm_mday + 1;
		if (DayStruct.day_nr == 32) {
		    DayStruct.day_nr = 1;
		    DayStruct.month_nr++;
		    if (DayStruct.month_nr == 13) {
			DayStruct.day_nr = 1;
			DayStruct.month_nr = 1;
		    }
		}
	    }
	    break;
	case SL_KEY_RIGHT:
	    DayStruct.day_nr++;
	    if (DayStruct.month_nr == (*TimeStruct).tm_mon + 1 &&
		DayStruct.day_nr == (*TimeStruct).tm_mday + 1)
		DayStruct.day_nr = (*TimeStruct).tm_mday;
	    if (DayStruct.day_nr == 32) {
		DayStruct.day_nr = 1;
		DayStruct.month_nr++;
		if (DayStruct.month_nr == 13) {
		    DayStruct.day_nr = 1;
		    DayStruct.month_nr = 1;
		}
	    }
	    break;
	}
    }
    SLsmg_gotorc(9, 61);
    SLsmg_set_color(COLOR_BOX_TITLE);
    SLsmg_printf("%s %i", MonthStr, DayStruct.day_nr);
    SLsmg_refresh();
}

void drawDate()
{
    int c = 0;
    char StartMonthStr[10];
    char StopMonthStr[10];
    char InputStartDay[] = "  \0", InputStartMonth[] = "  \0", InputStopDay[] = "  \0",
     InputStopMonth[] = "  \0";

    char MonthStr1[10], MonthStr2[10];

    while (c != '\t') {
	if (c == 'q' || c == 'Q')
	    finishWork();
	getMonthName(DateStruct.start_month, MonthStr1);
	getMonthName(DateStruct.stop_month, MonthStr2);
	SLsmg_set_color(COLOR_NORMAL);
	SLsmg_draw_clear_box(15, 57, 6, 21);
	SLsmg_gotorc(15, 61);
	SLsmg_set_color(COLOR_BOX_ACTIVE);
	SLsmg_printf("%s %02i - %s %02i", MonthStr1,
		   DateStruct.start_day, MonthStr2, DateStruct.stop_day);
	countConnections(DateStruct.start_day, DateStruct.start_month,
	       DateStruct.stop_day, DateStruct.stop_month, &DialSummary);
	SLsmg_set_color(COLOR_NORMAL);
	SLsmg_gotorc(16, 58);
	SLsmg_printf("%s: %i:%02i:%02i:%02i", _("Time"),
		     DialSummary.days, DialSummary.hours,
		     DialSummary.minutes, DialSummary.seconds);
	SLsmg_gotorc(17, 58);
	SLsmg_printf("%s: %i", _("Impulses"),
		     DialSummary.impulses);
	SLsmg_gotorc(18, 58);
	SLsmg_printf("%s: %.2f %s", _("Cost"),
		     DialSummary.price, _("$"));
	SLsmg_refresh();
	if (!VisualMonopolyInitiation)
	    c = SLkp_getkey_extend();
	else
	    c = '\t';
	if (c == SL_KEY_ENTER || c == 13) {
	    SLtt_set_cursor_visibility(1);
	    SLsmg_set_color(COLOR_NORMAL);
	    SLsmg_draw_clear_box(15, 57, 6, 21);
	    SLsmg_gotorc(17, 3);
	    SLsmg_write_char(SLSMG_RTEE_CHAR);
	    SLsmg_gotorc(15, 61);
	    SLsmg_set_color(COLOR_BOX_ACTIVE);
	    SLsmg_printf("%s", _("Input dates"));
	    SLsmg_set_color(COLOR_NORMAL);
	    SLsmg_gotorc(16, 58);
	    SLsmg_printf("%s: ", _("First day"));
	    SLkp_getnum(2, InputStartDay);
	    SLsmg_gotorc(17, 58);
	    SLsmg_printf("%s: ", _("First month"));
	    SLkp_getnum(2, InputStartMonth);
	    SLsmg_gotorc(18, 58);
	    SLsmg_printf("%s: ", _("Last day"));
	    SLkp_getnum(2, InputStopDay);
	    SLsmg_gotorc(19, 58);
	    SLsmg_printf("%s: ", _("Last month"));
	    SLkp_getnum(2, InputStopMonth);
	    DateStruct.start_day = atoi(InputStartDay);
	    DateStruct.start_month = atoi(InputStartMonth);
	    DateStruct.stop_day = atoi(InputStopDay);
	    DateStruct.stop_month = atoi(InputStopMonth);
	    SLtt_set_cursor_visibility(0);
	}
    }
    getMonthName(DateStruct.start_month, MonthStr1);
    getMonthName(DateStruct.stop_month, MonthStr2);
    SLsmg_gotorc(15, 61);
    SLsmg_set_color(COLOR_BOX_TITLE);
    SLsmg_printf("%s %02i - %s %02i", MonthStr1,
		 DateStruct.start_day, MonthStr2, DateStruct.stop_day);
    SLsmg_refresh();
}

void drawHeader()
{
    SLsmg_set_color(COLOR_NORMAL);
    SLsmg_draw_clear_box(0, 0, 3, 79);
    SLsmg_gotorc(1, (int) (79 - strlen(VERSION) - 24) / 2);
    SLsmg_printf(_("--== Monopoly ver. %s ==--"), VERSION);
    SLsmg_refresh();
}

void initV()
{
    loadBuffer(1);
    MonthStruct.month_nr = (*TimeStruct).tm_mon + 1;
    DayStruct.month_nr = (*TimeStruct).tm_mon + 1;
    DayStruct.day_nr = (*TimeStruct).tm_mday;
    DateStruct.start_day = (*TimeStruct).tm_mday - 1;
    DateStruct.start_month = (*TimeStruct).tm_mon + 1;
    DateStruct.stop_day = (*TimeStruct).tm_mday - 1;
    DateStruct.stop_month = (*TimeStruct).tm_mon + 1;
    ViewerStruct.LinesNr = getNumberOfLines(LogFileName);
    ViewerStruct.StartLine = ViewerStruct.LinesNr - 17;
    SLsmg_set_color(COLOR_NORMAL);
    SLtt_cls();
    drawViewer();
    drawMonth();
    drawDay();
    drawDate();
    drawHeader();
    drawStatus();
    VisualMonopolyInitiation = 0;
    ActiveWindow = 0;
    while (1) {
	ActiveWindow++;
	if (ActiveWindow == 5)
	    ActiveWindow = 1;
	drawStatus();
	switch (ActiveWindow) {
	case 4:
	    drawViewer();
	    break;
	case 1:
	    drawMonth();
	    break;
	case 2:
	    drawDay();
	    break;
	case 3:
	    drawDate();
	    break;
	}
    }
}


void createLogFile()
{
    updateLogFile(1);
}

void visualMonopoly()
{
    initV();
    SLkp_getkey_extend();
}

void printThisMonthStats()
{
    printf("%s.\n\r", _("This month calls"));
    countConnections(1, (*TimeStruct).tm_mon + 1, 33, (*TimeStruct).tm_mon + 1,
		     &DialSummary);
    printf("%s: %i:%02i:%02i:%02i\n", _("Time"), DialSummary.days,
	   DialSummary.hours, DialSummary.minutes, DialSummary.seconds);
    printf("%s: %i, %.2f %s\n\n", _("Impulses"),
	   DialSummary.impulses,
	   DialSummary.price, _("$"));
}

void printYesterdayStats()
{
    time_t tmp;
    struct tm *yesterday;

    tmp = mktime(TimeStruct) - 24 * 60 * 60;
    yesterday = localtime(&tmp);

    printf("%s.\n\r", _("Yesterday calls"));
    countConnections((*TimeStruct).tm_mday - 1, (*TimeStruct).tm_mon + 1,
      (*TimeStruct).tm_mday - 1, (*TimeStruct).tm_mon + 1, &DialSummary);

    printf("%s: %i:%02i:%02i:%02i\n", _("Time"), DialSummary.days,
	   DialSummary.hours, DialSummary.minutes, DialSummary.seconds);
    printf("%s: %i, %.2f %s\n\n", _("Impulses"),
	   DialSummary.impulses,
	   DialSummary.price, _("$"));
}

void printTodayStats()
{
    printf("%s.\n", _("Today calls"));
    countConnections((*TimeStruct).tm_mday, (*TimeStruct).tm_mon + 1,
	  (*TimeStruct).tm_mday, (*TimeStruct).tm_mon + 1, &DialSummary);
    printf("%s: %i:%02i:%02i:%02i\n", _("Time"), DialSummary.days,
	   DialSummary.hours, DialSummary.minutes, DialSummary.seconds);
    printf("%s: %i, %.2f %s\n\n", _("Impulses"),
	   DialSummary.impulses,
	   DialSummary.price, _("$"));
}

void printLastCallStats()
{
    printf("%s.\n", _("Last call"));
    countLastConnection(&DialSummary);
    printf("%s: %i:%02i:%02i:%02i\n", _("Time"), DialSummary.days,
	   DialSummary.hours, DialSummary.minutes, DialSummary.seconds);
    printf("%s: %i, %.2f %s\n\n", _("Impulses"),
	   DialSummary.impulses,
	   DialSummary.price, _("$"));

}

void usage()
{
    printf("%s\n", "monopoly [-n|-vd|-ctymd]");
    printf("       -v : %s\n", _("Monopoly with GUI"));
    printf("       -c : %s\n", _("Last call"));
    printf("       -t : %s\n", _("Today calls"));
    printf("       -y : %s\n", _("Yesterday calls"));
    printf("       -m : %s\n", _("This month calls"));
    printf("       -d : %s\n", _("don't update history file\n            (I will use actual history file!)."));
    printf("       -u : %s\n", _("update history file\n            (Nothing will be shown on terminal!)."));
    printf("       -n : %s\n\n", _("Create new history file"));
    configCleanup();
    chdir(CurrentDir);
    exit(0);
}

int main(int argc, char *argv[])
{
    struct tm *TS;
    char opt;
    int optflags;
/*  Set locale stuff */
    setlocale(LC_TIME, "");
    setlocale(LC_MESSAGES, "");
    bindtextdomain(PACKAGE, LOCALEDIR);
    textdomain(PACKAGE);

    time(&CurrentTime_InSec);
    TimeStruct = localtime(&CurrentTime_InSec);

/*  go into home directory     */
    getcwd(CurrentDir, 1000);
    chdir(getenv("HOME"));

    printf(_("--== Monopoly ver. %s ==--\n"), VERSION);

    readConfigFile();
/*SendConfigToMakeLog(ImpulsPrice, LogFileName, DelayTime); */

/*  Get all command line options and set flags     */
    optflags = 0;
    while ((opt = getopt(argc, argv, "udnvmyct")) != EOF) {
	switch (opt) {
	case 'u':
	    optflags |= FLAG_UPDATEONLY;
	    break;
	case 'd':
	    optflags |= FLAG_DONTUPDATE;
	    break;
	case 'n':
	    optflags |= FLAG_NEWLOG;
	    break;
	case 'v':
	    optflags |= FLAG_VISUAL;
	    break;
	case 'm':
	    optflags |= FLAG_MONTH;
	    break;
	case 'y':
	    optflags |= FLAG_YESTERDAY;
	    break;
	case 't':
	    optflags |= FLAG_TODAY;
	    break;
	case 'c':
	    optflags |= FLAG_LAST;
	    break;
	case '?':
	    optflags |= FLAG_ERROR;
	    usage();
	    break;
	}
    }

/*  Now use flags to run every part of the program in fixed order     */
    if (!optflags)
	usage();
    else {
	if ((optflags & FLAG_UPDATEONLY) == FLAG_UPDATEONLY)
	    updateLogFile(0);
	else {
	    if ((optflags & FLAG_NEWLOG) == FLAG_NEWLOG)
		createLogFile();
	    else if ((optflags & FLAG_DONTUPDATE) != FLAG_DONTUPDATE)
		updateLogFile(0);

	    if ((optflags & FLAG_VISUAL) == FLAG_VISUAL) {
		initSlang();
		initColor();
		visualMonopoly();
		deinitColor();
		deinitSlang();
	    }
	    if ((optflags & FLAG_MONTH) == FLAG_MONTH)
		printThisMonthStats();
	    if ((optflags & FLAG_YESTERDAY) == FLAG_YESTERDAY)
		printYesterdayStats();
	    if ((optflags & FLAG_TODAY) == FLAG_TODAY)
		printTodayStats();
	    if ((optflags & FLAG_LAST) == FLAG_LAST)
		printLastCallStats();
	}
    }
    configCleanup();
    chdir(CurrentDir);
    return 0;
}
